/******************************************************************************
 *{@C
 * Copyright:   2018-2022 Paul Obermeier (obermeier@poSoft.de)
 *
 *              See the file "Mawt_License.txt" for information on
 *              usage and redistribution of this file, and for a
 *              DISCLAIMER OF ALL WARRANTIES.
 *
 * Description: Main SWIG files for wrapping the FFMPEG library.
 *              This file is responsible for the Tcl package MAWT.
 *
 *****************************************************************************/

// The name of our generated Tcl package.

%module mawtLib


// Code to initialize the Tk stubs library.

%{
#include <tk.h>
%}

%init %{
#ifdef USE_TK_STUBS
    if (Tk_InitStubs(interp, (char*)"8.5", 0) == NULL) {
      return TCL_ERROR;
    }
#endif
%}

// Some typedefs as used in the vector and MAWT library.

typedef unsigned char UByte;
typedef unsigned int  Bool;


// Standard modules from the SWIG library

%include "typemaps.i"


// Common wrapper modules.

%include "vectors.i"
%include "tkphoto.i"


// Wrap the MAWT library.
%{
#include "mawtLib.h"
%}

%include "mawtLib.h"

// Wrap the ByteArray library.
// Note: This must be at the end of the include list, as this SWIG file
// redefines the standard typemaps for "unsigned char *" and
// "const unsigned char *".
// All other SWIG files rely on this standard mapping.

%include "bytearray.i"
